# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import uuid
from devtools_testutils.perfstress_tests import PerfStressTest
from azure.identity import DefaultAzureCredential
from azure.identity.aio import DefaultAzureCredential as AsyncDefaultAzureCredential
from azure.keyvault.administration import (
    KeyVaultAccessControlClient,
    KeyVaultDataAction,
    KeyVaultPermission,
    KeyVaultRoleScope,
)
from azure.keyvault.administration.aio import KeyVaultAccessControlClient as AsyncKeyVaultAccessControlClient


class GetRoleDefinitionTest(PerfStressTest):

    def __init__(self, arguments):
        super().__init__(arguments)

        # Auth configuration
        self.credential = DefaultAzureCredential()
        self.async_credential = AsyncDefaultAzureCredential()

        # Create clients
        vault_url = self.get_from_env("AZURE_MANAGEDHSM_URL")
        self.client = KeyVaultAccessControlClient(vault_url, self.credential, **self._client_kwargs)
        self.async_client = AsyncKeyVaultAccessControlClient(vault_url, self.async_credential, **self._client_kwargs)
        self.role_name = uuid.uuid4()
        self.scope = KeyVaultRoleScope.GLOBAL
        self.permissions = [KeyVaultPermission(data_actions=[KeyVaultDataAction.CREATE_HSM_KEY])]

    async def global_setup(self):
        """The global setup is run only once."""
        await super().global_setup()
        await self.async_client.set_role_definition(scope=self.scope, name=self.role_name, permissions=self.permissions)

    async def global_cleanup(self):
        """The global cleanup is run only once."""
        await self.async_client.delete_role_definition(scope=self.scope, name=self.role_name)
        await super().global_cleanup()

    async def close(self):
        """This is run after cleanup."""
        await self.async_client.close()
        await self.async_credential.close()
        await super().close()

    def run_sync(self):
        """The synchronous perf test."""
        self.client.get_role_definition(self.scope, self.role_name)

    async def run_async(self):
        """The asynchronous perf test."""
        await self.async_client.get_role_definition(self.scope, self.role_name)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import time
from functools import partial

import pytest
from azure.core.exceptions import ResourceExistsError
from azure.keyvault.administration import KeyVaultBackupClient
from azure.keyvault.administration._internal import parse_folder_url
from azure.keyvault.administration._internal.client_base import DEFAULT_VERSION
from devtools_testutils import recorded_by_proxy, set_bodiless_matcher

from _shared.test_case import KeyVaultTestCase
from _test_case import KeyVaultBackupClientPreparer, KeyVaultBackupClientSasPreparer, get_decorator

all_api_versions = get_decorator()
only_default = get_decorator(api_versions=[DEFAULT_VERSION])


class TestBackupClientTests(KeyVaultTestCase):

    def create_key_client(self, vault_uri, **kwargs):
        from azure.keyvault.keys import KeyClient

        credential = self.get_credential(KeyClient)
        return self.create_client_from_credential(KeyClient, credential=credential, vault_url=vault_uri, **kwargs)

    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy
    def test_full_backup_and_restore(self, client, **kwargs):
        set_bodiless_matcher()
        # backup the vault
        container_uri = kwargs.pop("container_uri")
        # make sure an error isn't raised by pre-backup check; i.e. ensure the backup can be done
        client.begin_pre_backup(container_uri, use_managed_identity=True).wait()
        backup_poller = client.begin_backup(container_uri, use_managed_identity=True)
        backup_operation = backup_poller.result()
        assert backup_operation.folder_url

        if self.is_live:
            # Additional waiting to ensure backup will be available for restore. Otherwise, can get the following:
            # "This Managed HSM hasn't been backed up within the safe time limit of 30 minutes"
            time.sleep(15)

        # restore the backup
        # make sure an error isn't raised by pre-restore check; i.e. ensure the restore can be done
        client.begin_pre_restore(backup_operation.folder_url, use_managed_identity=True).wait()
        restore_poller = client.begin_restore(backup_operation.folder_url, use_managed_identity=True)
        restore_poller.wait()
        if self.is_live:
            time.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy
    def test_full_backup_and_restore_rehydration(self, client, **kwargs):
        set_bodiless_matcher()
        container_uri = kwargs.pop("container_uri")

        # backup the vault
        backup_poller = client.begin_backup(blob_storage_url=container_uri, use_managed_identity=True)

        # create a new poller from a continuation token
        token = backup_poller.continuation_token()
        rehydrated = client.begin_backup(container_uri, use_managed_identity=True, continuation_token=token)

        rehydrated_operation = rehydrated.result()
        assert rehydrated_operation.folder_url
        backup_operation = backup_poller.result()
        assert backup_operation.folder_url == rehydrated_operation.folder_url

        if self.is_live:
            time.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the backup
        restore_poller = client.begin_restore(folder_url=backup_operation.folder_url, use_managed_identity=True)

        # create a new poller from a continuation token
        token = restore_poller.continuation_token()
        rehydrated = client.begin_restore(
            backup_operation.folder_url, use_managed_identity=True, continuation_token=token
        )

        rehydrated.wait()
        restore_poller.wait()
        if self.is_live:
            time.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy
    def test_selective_key_restore(self, client, **kwargs):
        # create a key to selectively restore
        managed_hsm_url = kwargs.pop("managed_hsm_url")
        key_client = self.create_key_client(managed_hsm_url)
        key_name = self.get_resource_name("selective-restore-test-key")
        key_client.create_rsa_key(key_name)

        # backup the vault
        container_uri = kwargs.pop("container_uri")
        backup_poller = client.begin_backup(container_uri, use_managed_identity=True)
        backup_operation = backup_poller.result()

        if self.is_live:
            time.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the key
        restore_poller = client.begin_restore(backup_operation.folder_url, use_managed_identity=True, key_name=key_name)
        restore_poller.wait()

        # delete the key
        delete_function = partial(key_client.begin_delete_key, key_name)
        delete_poller = self._poll_until_no_exception(delete_function, ResourceExistsError)
        delete_poller.wait()
        key_client.purge_deleted_key(key_name)
        if self.is_live:
            time.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientPreparer()
    @recorded_by_proxy
    def test_backup_client_polling(self, client, **kwargs):
        set_bodiless_matcher()

        # backup the vault
        container_uri = kwargs.pop("container_uri")
        backup_poller = client.begin_backup(container_uri, use_managed_identity=True)

        # create a new poller from a continuation token
        token = backup_poller.continuation_token()
        rehydrated = client.begin_backup(container_uri, use_managed_identity=True, continuation_token=token)

        # check that pollers and polling methods behave as expected
        if self.is_live:
            assert backup_poller.status() == "InProgress"
            assert not backup_poller.done() or backup_poller.polling_method().finished()
            assert rehydrated.status() == "InProgress"
            assert not rehydrated.done() or rehydrated.polling_method().finished()

        backup_operation = backup_poller.result()
        assert backup_poller.status() == "Succeeded" and backup_poller.polling_method().status() == "Succeeded"
        rehydrated_operation = rehydrated.result()
        assert rehydrated.status() == "Succeeded" and rehydrated.polling_method().status() == "Succeeded"
        assert backup_operation.folder_url == rehydrated_operation.folder_url

        # rehydrate a poller with a continuation token of a completed operation
        late_rehydrated = client.begin_backup(container_uri, use_managed_identity=True, continuation_token=token)
        assert late_rehydrated.status() == "Succeeded"
        late_rehydrated.wait()

        if self.is_live:
            time.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the backup
        restore_poller = client.begin_restore(backup_operation.folder_url, use_managed_identity=True)

        # create a new poller from a continuation token
        token = restore_poller.continuation_token()
        rehydrated = client.begin_restore(
            backup_operation.folder_url, use_managed_identity=True, continuation_token=token
        )

        # check that pollers and polling methods behave as expected
        if self.is_live:
            assert restore_poller.status() == "InProgress"
            assert not restore_poller.done() or restore_poller.polling_method().finished()
            assert rehydrated.status() == "InProgress"
            assert not rehydrated.done() or rehydrated.polling_method().finished()

        rehydrated.wait()
        assert rehydrated.status() == "Succeeded" and rehydrated.polling_method().status() == "Succeeded"
        restore_poller.wait()
        assert restore_poller.status() == "Succeeded" and restore_poller.polling_method().status() == "Succeeded"

        if self.is_live:
            time.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.live_test_only
    @pytest.mark.parametrize("api_version", only_default)
    @KeyVaultBackupClientSasPreparer()
    def test_backup_restore_sas(self, client: KeyVaultBackupClient, **kwargs):
        # backup the vault
        container_uri = kwargs.pop("container_uri")
        sas_token = kwargs.pop("sas_token")

        if self.is_live and not sas_token:
            pytest.skip(
                "SAS token is required for live tests. Please set the BLOB_STORAGE_SAS_TOKEN environment variable."
            )

        client.begin_pre_backup(container_uri, sas_token=sas_token).wait()
        backup_poller = client.begin_backup(container_uri, sas_token)  # Test positional SAS token for backwards compat
        backup_operation = backup_poller.result()
        assert backup_operation.folder_url

        if self.is_live:
            time.sleep(15)  # Additional waiting to ensure backup will be available for restore

        # restore the backup
        client.begin_pre_restore(backup_operation.folder_url, sas_token=sas_token).wait()
        restore_poller = client.begin_restore(backup_operation.folder_url, sas_token)  # Test positional SAS token
        restore_poller.wait()
        if self.is_live:
            time.sleep(60)  # additional waiting to avoid conflicts with resources in other tests


@pytest.mark.parametrize(
    "url,expected_container_url,expected_folder_name",
    [
        (
            "https://account.blob.core.windows.net/backup/mhsm-account-2020090117323313",
            "https://account.blob.core.windows.net/backup",
            "mhsm-account-2020090117323313",
        ),
        ("https://account.storage/account/storage", "https://account.storage/account", "storage"),
        ("https://account.storage/a/b/c", "https://account.storage/a", "b/c"),
        ("https://account.storage/a/b-c", "https://account.storage/a", "b-c"),
    ],
)
def test_parse_folder_url(url, expected_container_url, expected_folder_name):
    container_url, folder_name = parse_folder_url(url)
    assert container_url == expected_container_url
    assert folder_name == expected_folder_name



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
Tests for the HTTP challenge authentication implementation.
"""

import functools
import time
from unittest.mock import Mock
from uuid import uuid4

import pytest
from azure.core.credentials import AccessToken, AccessTokenInfo
from azure.core.pipeline import AsyncPipeline, Pipeline
from azure.core.rest import HttpRequest
from azure.keyvault.administration._internal import ChallengeAuthPolicy, HttpChallengeCache
from azure.keyvault.administration._internal.async_challenge_auth_policy import AsyncChallengeAuthPolicy

TOKEN_TYPES = [AccessToken, AccessTokenInfo]


def empty_challenge_cache(fn):
    @functools.wraps(fn)
    def wrapper(**kwargs):
        HttpChallengeCache.clear()
        assert len(HttpChallengeCache._cache) == 0
        return fn(**kwargs)

    return wrapper


def async_empty_challenge_cache(fn):
    @functools.wraps(fn)
    async def wrapper(**kwargs):
        HttpChallengeCache.clear()
        assert len(HttpChallengeCache._cache) == 0
        return await fn(**kwargs)

    return wrapper


def get_random_url():
    """The challenge cache is keyed on URLs. Random URLs defend against tests interfering with each other."""

    return f"https://{uuid4()}.vault.azure.net/{uuid4()}".replace("-", "")


@empty_challenge_cache
@pytest.mark.parametrize("token_type", TOKEN_TYPES)
def test_request_body_not_reused_across_requests(token_type):
    """A request's body must not leak into a later request made by the same client.

    Regression test for the replay bug: the original request used to be stashed on the policy instance and was
    never cleared, so a subsequent bodiless request (e.g. a polling GET during a backup/restore operation) that
    triggered its own challenge would have the earlier request's body (and method/URL) replayed onto it. The copy
    is now stored per-request on the pipeline context, so it cannot leak across requests. See
    https://github.com/Azure/azure-sdk-for-python/pull/47742.
    """

    expected_token = "expected_token"
    first_content = b"a duck"
    first_url = get_random_url()
    second_url = get_random_url()
    challenge = Mock(
        status_code=401,
        headers={
            "WWW-Authenticate": 'Bearer authorization="https://authority.net/tenant", resource=https://vault.azure.net'
        },
    )

    class Requests:
        count = 0

    def send(request):
        Requests.count += 1
        if Requests.count == 1:
            # first request (POST with body): the body is stripped to elicit a challenge
            assert not request.body
            assert request.headers["Content-Length"] == "0"
            return challenge
        elif Requests.count == 2:
            # first request is retried with its original body and authorization
            assert request.body == first_content
            assert expected_token in request.headers["Authorization"]
            return Mock(status_code=200)
        elif Requests.count == 3:
            # second request (bodiless GET): elicits its own challenge and must have no body
            assert not request.body
            return challenge
        elif Requests.count == 4:
            # second request is retried: it must remain a bodiless GET, i.e. the first request's body and
            # method/URL must NOT be replayed onto it
            assert not request.body
            assert request.method == "GET"
            assert request.url == second_url
            assert expected_token in request.headers["Authorization"]
            return Mock(status_code=200)
        raise ValueError("unexpected request")

    def get_token(*_, **__):
        return token_type(expected_token, time.time() + 3600)

    if token_type == AccessToken:
        credential = Mock(spec_set=["get_token"], get_token=Mock(wraps=get_token))
    else:
        credential = Mock(spec_set=["get_token_info"], get_token_info=Mock(wraps=get_token))

    # a single policy instance handles both requests; the fix prevents state from one request leaking into the next
    policy = ChallengeAuthPolicy(credential=credential)
    pipeline = Pipeline(policies=[policy], transport=Mock(send=send))

    first_request = HttpRequest("POST", first_url)
    first_request.set_bytes_body(first_content)
    pipeline.run(first_request)

    pipeline.run(HttpRequest("GET", second_url))


@pytest.mark.asyncio
@async_empty_challenge_cache
@pytest.mark.parametrize("token_type", TOKEN_TYPES)
async def test_request_body_not_reused_across_requests_async(token_type):
    """A request's body must not leak into a later request made by the same client (async).

    Regression test for the replay bug: the original request used to be stashed on the policy instance and was
    never cleared, so a subsequent bodiless request (e.g. a polling GET during a backup/restore operation) that
    triggered its own challenge would have the earlier request's body (and method/URL) replayed onto it. The copy
    is now stored per-request on the pipeline context, so it cannot leak across requests. See
    https://github.com/Azure/azure-sdk-for-python/pull/47742.
    """

    expected_token = "expected_token"
    first_content = b"a duck"
    first_url = get_random_url()
    second_url = get_random_url()
    challenge = Mock(
        status_code=401,
        headers={
            "WWW-Authenticate": 'Bearer authorization="https://authority.net/tenant", resource=https://vault.azure.net'
        },
    )

    class Requests:
        count = 0

    async def send(request):
        Requests.count += 1
        if Requests.count == 1:
            # first request (POST with body): the body is stripped to elicit a challenge
            assert not request.body
            assert request.headers["Content-Length"] == "0"
            return challenge
        elif Requests.count == 2:
            # first request is retried with its original body and authorization
            assert request.body == first_content
            assert expected_token in request.headers["Authorization"]
            return Mock(status_code=200)
        elif Requests.count == 3:
            # second request (bodiless GET): elicits its own challenge and must have no body
            assert not request.body
            return challenge
        elif Requests.count == 4:
            # second request is retried: it must remain a bodiless GET, i.e. the first request's body and
            # method/URL must NOT be replayed onto it
            assert not request.body
            assert request.method == "GET"
            assert request.url == second_url
            assert expected_token in request.headers["Authorization"]
            return Mock(status_code=200)
        raise ValueError("unexpected request")

    async def get_token(*_, **__):
        return token_type(expected_token, time.time() + 3600)

    if token_type == AccessToken:
        credential = Mock(spec_set=["get_token"], get_token=Mock(wraps=get_token))
    else:
        credential = Mock(spec_set=["get_token_info"], get_token_info=Mock(wraps=get_token))

    # a single policy instance handles both requests; the fix prevents state from one request leaking into the next
    policy = AsyncChallengeAuthPolicy(credential=credential)
    pipeline = AsyncPipeline(policies=[policy], transport=Mock(send=send))

    first_request = HttpRequest("POST", first_url)
    first_request.set_bytes_body(first_content)
    await pipeline.run(first_request)

    await pipeline.run(HttpRequest("GET", second_url))



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import os
import base64
import asyncio

from azure.identity.aio import DefaultAzureCredential
from azure.keyvault.administration import KeyVaultEkmConnection
from azure.keyvault.administration.aio import KeyVaultEkmClient

# ----------------------------------------------------------------------------------------------------------
# Prerequisites:
# 1. A managed HSM (https://learn.microsoft.com/azure/key-vault/managed-hsm/quick-create-cli)
#
# 2. azure-keyvault-administration and azure-identity libraries (pip install these)
#
# 3. Set environment variable MANAGED_HSM_URL with the URL of your managed HSM, EKM_PROXY_HOST with the host of your EKM proxy,
#    and CA_CERTIFICATE with the proxy server's certificate in DER format and base64 encoded.
#
# 4. Set up your environment to use azure-identity's DefaultAzureCredential. For more information about how to configure
#    the DefaultAzureCredential, refer to https://aka.ms/azsdk/python/identity/docs#azure.identity.DefaultAzureCredential
#
# 5. An EKM (External Key Manager) proxy reachable from the Managed HSM. Replace the placeholder host
#    and certificate bytes below with values that match your EKM proxy deployment.
#
# ----------------------------------------------------------------------------------------------------------
# Sample - demonstrates Managed HSM External Key Manager (EKM) connection management
#
# 1. Create an EKM connection (create_ekm_connection)
#
# 2. Read the EKM connection (get_ekm_connection)
#
# 3. Get the EKM proxy client certificate (get_ekm_certificate)
#
# 4. Check an EKM connection (check_ekm_connection)
#
# 5. Update the EKM connection (update_ekm_connection)
#
# 6. Delete the EKM connection (delete_ekm_connection)
# ----------------------------------------------------------------------------------------------------------

# Instantiate an EKM client that will be used to call the service.
# Here we use the DefaultAzureCredential, but any azure-identity credential can be used.


async def run_sample():
    MANAGED_HSM_URL = os.environ["MANAGED_HSM_URL"]
    credential = DefaultAzureCredential()
    client = KeyVaultEkmClient(vault_url=MANAGED_HSM_URL, credential=credential)

    # First, let's create an EKM connection
    print("\n.. Create EKM connection")
    EKM_PROXY_HOST = os.environ["EKM_PROXY_HOST"]
    CA_CERTIFICATE = os.environ["CA_CERTIFICATE"]
    CA_CERTIFICATES = [CA_CERTIFICATE]
    ekm_connection = KeyVaultEkmConnection(
        host=EKM_PROXY_HOST,
        server_ca_certificates=[base64.b64decode(cert) for cert in CA_CERTIFICATES],
        path_prefix="/api/v1",
    )
    created_ekm_connection = await client.create_ekm_connection(connection=ekm_connection)
    print(f"EKM connection created with host: {created_ekm_connection.host}")

    # Let's get the EKM connection we just created
    print("\n.. Get EKM connection")
    retrieved_ekm_connection = await client.get_ekm_connection()
    print("Retrieved EKM connection with:")
    print(f"\tHost: {retrieved_ekm_connection.host}")
    print(f"\tPath prefix: {retrieved_ekm_connection.path_prefix}")
    print(f"\tServer subject common name: {retrieved_ekm_connection.server_subject_common_name}")

    # Get the EKM certificate
    print("\n.. Get EKM certificate")
    ekm_certificate = await client.get_ekm_certificate()
    print(f"EKM certificate retrieved with subject: {ekm_certificate.subject_common_name}")

    # Check the EKM connection status
    print("\n.. Check EKM connection")
    connection_status = await client.check_ekm_connection()
    print("EKM connection status:")
    print(f"\tAPI Version: {connection_status.api_version}")
    print(f"\tProxy Vendor: {connection_status.proxy_vendor}")
    print(f"\tProxy Name: {connection_status.proxy_name}")
    print(f"\tEKM Vendor: {connection_status.ekm_vendor}")
    print(f"\tEKM Product: {connection_status.ekm_product}")

    # Update the EKM connection
    print("\n.. Update EKM connection")
    updated_ekm_connection = KeyVaultEkmConnection(
        host="ekm-proxy-updated.contoso.com",
        server_ca_certificates=[base64.b64decode(cert) for cert in CA_CERTIFICATES],
        path_prefix="/api/v2",
    )
    result = await client.update_ekm_connection(connection=updated_ekm_connection)
    print(f"EKM connection updated with host: {result.host}")

    # Finally, let's delete the EKM connection
    print("\n.. Delete EKM connection")
    deleted_ekm_connection = await client.delete_ekm_connection()
    print("EKM connection deleted successfully")

    await client.close()
    await credential.close()


if __name__ == "__main__":
    asyncio.run(run_sample())



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from typing import Any

from azure.core.tracing.decorator import distributed_trace

from ._internal import KeyVaultClientBase
from ._models import KeyVaultEkmConnection, KeyVaultEkmProxyClientCertificateInfo, KeyVaultEkmProxyInfo


class KeyVaultEkmClient(KeyVaultClientBase):
    """Provides methods to manage Managed HSM External Key Manager (EKM) connections.

    :param str vault_url: URL of the vault on which the client will operate. This is also called the vault's "DNS Name".
        You should validate that this URL references a valid Key Vault or Managed HSM resource.
        See https://aka.ms/azsdk/blog/vault-uri for details.
    :param credential: An object which can provide an access token for the vault, such as a credential from
        :mod:`azure.identity`
    :type credential: ~azure.core.credentials.TokenCredential

    :keyword api_version: Version of the service API to use. EKM operations require service API version
        ``2026-01-01-preview`` or later.
    :paramtype api_version: ~azure.keyvault.administration.ApiVersion or str
    :keyword bool verify_challenge_resource: Whether to verify the authentication challenge resource matches the Key
        Vault or Managed HSM domain. Defaults to True.
    """

    # pylint:disable=protected-access

    @distributed_trace
    def get_ekm_connection(self, **kwargs: Any) -> KeyVaultEkmConnection:
        """Gets the configured EKM connection.

        :returns: The configured EKM connection.
        :rtype: ~azure.keyvault.administration.KeyVaultEkmConnection
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = self._client.get_ekm_connection(**kwargs)
        return KeyVaultEkmConnection._from_generated(result)

    @distributed_trace
    def create_ekm_connection(self, connection: KeyVaultEkmConnection, **kwargs: Any) -> KeyVaultEkmConnection:
        """Creates the EKM connection.

        If an EKM connection already exists, this operation fails.

        :param connection: The EKM connection to create.
        :type connection: ~azure.keyvault.administration.KeyVaultEkmConnection

        :returns: The created EKM connection.
        :rtype: ~azure.keyvault.administration.KeyVaultEkmConnection
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = self._client.create_ekm_connection(ekm_connection=connection._to_generated(), **kwargs)
        return KeyVaultEkmConnection._from_generated(result)

    @distributed_trace
    def update_ekm_connection(self, connection: KeyVaultEkmConnection, **kwargs: Any) -> KeyVaultEkmConnection:
        """Updates the existing EKM connection.

        If no EKM connection exists, this operation fails.

        :param connection: The EKM connection to update.
        :type connection: ~azure.keyvault.administration.KeyVaultEkmConnection

        :returns: The updated EKM connection.
        :rtype: ~azure.keyvault.administration.KeyVaultEkmConnection
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = self._client.update_ekm_connection(ekm_connection=connection._to_generated(), **kwargs)
        return KeyVaultEkmConnection._from_generated(result)

    @distributed_trace
    def delete_ekm_connection(  # pylint:disable=bad-option-value,delete-operation-wrong-return-type
        self, **kwargs: Any
    ) -> KeyVaultEkmConnection:
        """Deletes the existing EKM connection.

        If no EKM connection exists, this operation fails.

        :returns: The deleted EKM connection.
        :rtype: ~azure.keyvault.administration.KeyVaultEkmConnection
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = self._client.delete_ekm_connection(**kwargs)
        return KeyVaultEkmConnection._from_generated(result)

    @distributed_trace
    def get_ekm_certificate(self, **kwargs: Any) -> KeyVaultEkmProxyClientCertificateInfo:
        """Gets the EKM proxy client certificate information used to authenticate to the EKM proxy.

        :returns: The EKM proxy client certificate information.
        :rtype: ~azure.keyvault.administration.KeyVaultEkmProxyClientCertificateInfo
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = self._client.get_ekm_certificate(**kwargs)
        return KeyVaultEkmProxyClientCertificateInfo._from_generated(result)

    @distributed_trace
    def check_ekm_connection(self, **kwargs: Any) -> KeyVaultEkmProxyInfo:
        """Checks the EKM connection by pinging the EKM proxy.

        :returns: Information about the EKM proxy returned by the connectivity check.
        :rtype: ~azure.keyvault.administration.KeyVaultEkmProxyInfo
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        result = self._client.check_ekm_connection(**kwargs)
        return KeyVaultEkmProxyInfo._from_generated(result)

    def __enter__(self) -> "KeyVaultEkmClient":
        self._client.__enter__()
        return self



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See LICENSE.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import os
from unittest import mock


import pytest
from devtools_testutils import (
    add_general_string_sanitizer,
    add_oauth_response_sanitizer,
    add_uri_regex_sanitizer,
    is_live,
    remove_batch_sanitizers,
    set_custom_default_matcher,
)

os.environ["PYTHONHASHSEED"] = "0"


@pytest.fixture(scope="session", autouse=True)
def add_sanitizers(test_proxy):
    set_custom_default_matcher(compare_bodies=False, ignore_query_ordering=True, ignored_headers="Accept")
    azure_keyvault_url = os.getenv("AZURE_KEYVAULT_URL", "https://Sanitized.vault.azure.net")
    azure_keyvault_url = azure_keyvault_url.rstrip("/")
    keyvault_tenant_id = os.getenv("KEYVAULT_TENANT_ID", "keyvault_tenant_id")
    keyvault_subscription_id = os.getenv("KEYVAULT_SUBSCRIPTION_ID", "keyvault_subscription_id")
    azure_managedhsm_url = os.environ.get("AZURE_MANAGEDHSM_URL", "https://Sanitized.managedhsm.azure.net")
    azure_managedhsm_url = azure_managedhsm_url.rstrip("/")
    azure_attestation_uri = os.environ.get("AZURE_KEYVAULT_ATTESTATION_URL", "https://Sanitized.azurewebsites.net")
    azure_attestation_uri = azure_attestation_uri.rstrip("/")
    storage_url = os.environ.get("BLOB_STORAGE_URL", "https://Sanitized.blob.core.windows.net")
    client_id = os.environ.get("KEYVAULT_CLIENT_ID", "service-principal-id")
    sas_token = os.environ.get("BLOB_STORAGE_SAS_TOKEN", "fake-sas")
    ekm_host = os.environ.get("EKM_PROXY_HOST", "fake-ekm-host")
    ekm_certificate = os.environ.get("EKM_SERVER_CA_CERTIFICATE", "ZmFrZS1jZXJ0LWRhdGE=")

    add_general_string_sanitizer(target=azure_keyvault_url, value="https://Sanitized.vault.azure.net")
    add_general_string_sanitizer(target=keyvault_tenant_id, value="00000000-0000-0000-0000-000000000000")
    add_general_string_sanitizer(target=keyvault_subscription_id, value="00000000-0000-0000-0000-000000000000")
    add_general_string_sanitizer(target=azure_managedhsm_url, value="https://Sanitized.managedhsm.azure.net")
    add_general_string_sanitizer(target=azure_attestation_uri, value="https://Sanitized.azurewebsites.net")
    add_general_string_sanitizer(target=storage_url, value="https://Sanitized.blob.core.windows.net")
    add_general_string_sanitizer(target=sas_token, value="fake-sas")
    add_general_string_sanitizer(target=ekm_host, value="fake-ekm-host")
    add_general_string_sanitizer(target=ekm_certificate, value="ZmFrZS1jZXJ0LWRhdGE=")
    add_general_string_sanitizer(target=client_id, value="service-principal-id")
    # Sanitize API versions of `azure-keyvault-keys` requests
    add_uri_regex_sanitizer(
        regex="keys/([^/]*)/create\\?api-version=(\\S*)", value="keys/$1/create?api-version=sanitized"
    )
    add_uri_regex_sanitizer(regex="keys/([^/]*)\\?api-version=(\\S*)", value="keys/$1?api-version=sanitized")
    add_oauth_response_sanitizer()

    # Remove the following sanitizers since certain fields are needed in tests and are non-sensitive:
    #  - AZSDK3430: $..id
    #  - AZSDK3493: $..name
    remove_batch_sanitizers(["AZSDK3430", "AZSDK3493"])


@pytest.fixture(scope="session", autouse=True)
def patch_async_sleep():
    async def immediate_return(_):
        return

    if not is_live():
        with mock.patch("asyncio.sleep", immediate_return):
            yield

    else:
        yield


@pytest.fixture(scope="session", autouse=True)
def patch_sleep():
    def immediate_return(_):
        return

    if not is_live():
        with mock.patch("time.sleep", immediate_return):
            yield

    else:
        yield



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from typing import Any, Dict, List, Optional, Union

from azure.core.rest import HttpResponse

from ._enums import KeyVaultSettingType
from ._generated.models import (
    EkmConnection,
    EkmProxyClientCertificateInfo,
    EkmProxyInfo,
    FullBackupOperation,
    Permission,
    RoleAssignment,
    RoleAssignmentProperties,
    RoleAssignmentPropertiesWithScope,
    RoleDefinition,
    Setting,
)


class KeyVaultPermission(object):
    """Role definition permissions.

    :ivar list[str] actions: Action permissions that are granted.
    :ivar list[str] not_actions: Action permissions that are excluded but not denied. They may be granted by other role
     definitions assigned to a principal.
    :ivar list[str] data_actions: Data action permissions that are granted.
    :ivar list[str] not_data_actions: Data action permissions that are excluded but not denied. They may be granted by
     other role definitions assigned to a principal.
    """

    def __init__(self, **kwargs: Any) -> None:
        self.actions = kwargs.get("actions")
        self.not_actions = kwargs.get("not_actions")
        self.data_actions = kwargs.get("data_actions")
        self.not_data_actions = kwargs.get("not_data_actions")

    @classmethod
    def _from_generated(cls, permissions: Permission) -> "KeyVaultPermission":
        return cls(
            actions=permissions.actions,
            not_actions=permissions.not_actions,
            data_actions=permissions.data_actions,
            not_data_actions=permissions.not_data_actions,
        )


class KeyVaultRoleAssignment(object):
    """Represents the assignment to a principal of a role over a scope

    :ivar str name: the assignment's name
    :ivar KeyVaultRoleAssignmentProperties properties: the assignment's properties
    :ivar str role_assignment_id: unique identifier for the assignment
    :ivar str type: type of the assignment
    """

    def __init__(self, **kwargs: Any) -> None:
        self.name = kwargs.get("name")
        self.properties = kwargs.get("properties")
        self.role_assignment_id = kwargs.get("role_assignment_id")
        self.type = kwargs.get("assignment_type")

    def __repr__(self) -> str:
        return f"KeyVaultRoleAssignment<{self.role_assignment_id}>"

    @classmethod
    def _from_generated(cls, role_assignment: RoleAssignment) -> "KeyVaultRoleAssignment":
        # pylint:disable=protected-access
        return cls(
            role_assignment_id=role_assignment.id,
            name=role_assignment.name,
            assignment_type=role_assignment.type,
            properties=(
                KeyVaultRoleAssignmentProperties._from_generated(role_assignment.properties)
                if role_assignment.properties
                else KeyVaultRoleAssignmentProperties()
            ),
        )


class KeyVaultRoleAssignmentProperties(object):
    """Properties of a role assignment

    :ivar str principal_id: ID of the principal the assignment applies to. This maps to an Active Directory user,
        service principal, or security group.
    :ivar str role_definition_id: ID of the scope's role definition
    :ivar str scope: the scope of the assignment
    """

    def __init__(self, **kwargs: Any) -> None:
        self.principal_id = kwargs.get("principal_id")
        self.role_definition_id = kwargs.get("role_definition_id")
        self.scope = kwargs.get("scope")

    def __repr__(self) -> str:
        string = (
            f"KeyVaultRoleAssignmentProperties(principal_id={self.principal_id}, "
            + f"role_definition_id={self.role_definition_id}, scope={self.scope})"
        )
        return string[:1024]

    @classmethod
    def _from_generated(
        cls, role_assignment_properties: Union[RoleAssignmentProperties, RoleAssignmentPropertiesWithScope]
    ) -> "KeyVaultRoleAssignmentProperties":
        # the generated RoleAssignmentProperties and RoleAssignmentPropertiesWithScope
        # models differ only in that the latter has a "scope" attribute
        return cls(
            principal_id=role_assignment_properties.principal_id,
            role_definition_id=role_assignment_properties.role_definition_id,
            scope=getattr(role_assignment_properties, "scope", None),
        )


class KeyVaultRoleDefinition(object):
    """The definition of a role over one or more scopes

    :ivar list[str] assignable_scopes: scopes the role can be assigned over
    :ivar str description: description of the role definition
    :ivar str id: unique identifier for this role definition
    :ivar str name: the role definition's name
    :ivar list[KeyVaultPermission] permissions: permissions defined for the role
    :ivar str role_name: the role's name
    :ivar str role_type: type of the role
    :ivar str type: type of the role definition
    """

    def __init__(self, **kwargs: Any) -> None:
        self.assignable_scopes = kwargs.get("assignable_scopes")
        self.description = kwargs.get("description")
        self.id = kwargs.get("id")
        self.name = kwargs.get("name")
        self.permissions = kwargs.get("permissions")
        self.role_name = kwargs.get("role_name")
        self.role_type = kwargs.get("role_type")
        self.type = kwargs.get("type")

    def __repr__(self) -> str:
        return f"KeyVaultRoleDefinition<{self.id}>"

    @classmethod
    def _from_generated(cls, definition: RoleDefinition) -> "KeyVaultRoleDefinition":
        # pylint:disable=protected-access
        return cls(
            assignable_scopes=definition.properties.assignable_scopes if definition.properties else None,
            description=definition.properties.description if definition.properties else None,
            id=definition.id,
            name=definition.name,
            permissions=(
                [KeyVaultPermission._from_generated(p) for p in definition.properties.permissions or []]
                if definition.properties
                else None
            ),
            role_name=definition.properties.role_name if definition.properties else None,
            role_type=definition.properties.role_type if definition.properties else None,
            type=definition.type,
        )


class KeyVaultBackupResult(object):
    """A Key Vault full backup operation result

    :ivar str folder_url: URL of the Azure Blob Storage container containing the backup
    """

    # pylint:disable=unused-argument

    def __init__(self, **kwargs: Any) -> None:
        self.folder_url: Optional[str] = kwargs.get("folder_url")

    @classmethod
    def _from_generated(
        cls, response: HttpResponse, deserialized_operation: FullBackupOperation, response_headers: Dict
    ) -> "KeyVaultBackupResult":
        return cls(folder_url=deserialized_operation.azure_storage_blob_container_uri)


class KeyVaultSetting(object):
    """A Key Vault setting.

    :ivar str name: The name of the account setting.
    :ivar str value: The value of the account setting.
    :ivar setting_type: The type specifier of the value.
    :vartype setting_type: str or KeyVaultSettingType or None

    :param str name: The name of the account setting.
    :param str value: The value of the account setting.
    :param setting_type: The type specifier of the value.
    :type setting_type: str or KeyVaultSettingType or None
    """

    def __init__(
        self,
        name: str,
        value: Union[str, bool],
        setting_type: Optional[Union[str, KeyVaultSettingType]] = None,
        **kwargs,  # pylint:disable=unused-argument
    ) -> None:
        self.name = name
        self.value = value if isinstance(value, str) else str(value)  # `value` is stored as a string
        if setting_type == KeyVaultSettingType.BOOLEAN:
            self.setting_type: Optional[Union[str, KeyVaultSettingType]] = KeyVaultSettingType.BOOLEAN
        else:
            self.setting_type = setting_type.lower() if isinstance(setting_type, str) else setting_type

        # If a setting type isn't provided, set it based on `value`'s type (without inferring from the value itself)
        if self.setting_type is None:
            if isinstance(value, bool):
                self.setting_type = KeyVaultSettingType.BOOLEAN

        # If the setting is a boolean, lower-case the string for serialization
        if self.setting_type == KeyVaultSettingType.BOOLEAN:
            self.value = self.value.lower()

    def getboolean(self) -> bool:
        """Gets the account setting value as a boolean if the ``setting_type`` is ``KeyVaultSettingType.BOOLEAN``.

        :returns: The account setting value as a boolean.
        :rtype: bool

        :raises ValueError: if the ``setting_type`` is not boolean or the value cannot be represented as a boolean.
        """
        if self.setting_type == KeyVaultSettingType.BOOLEAN:
            if self.value == "true":
                return True
            if self.value == "false":
                return False
        raise ValueError(
            'The `setting_type` of the setting must be `KeyVaultSettingType.BOOLEAN` and the `value` must be "true" '
            'or "false" in order to use `getboolean`.'
        )

    @classmethod
    def _from_generated(cls, setting: Setting) -> "KeyVaultSetting":
        setting_type = KeyVaultSettingType.BOOLEAN if setting.type == "boolean" else setting.type
        return cls(name=setting.name, value=setting.value, setting_type=setting_type)


class KeyVaultEkmConnection(object):
    """An External Key Manager (EKM) connection.

    :param str host: EKM proxy FQDN (Fully Qualified Domain Name). Only allowed characters are
        ``a-z``, ``A-Z``, ``0-9``, hyphen (``-``), dot (``.``), and colon (``:``).
    :param server_ca_certificates: The root CA certificate chain that issued the proxy server's
        certificate. An array of certificates in the certificate chain, each in DER format and
        base64 encoded.
    :type server_ca_certificates: list[bytes]

    :keyword str path_prefix: Optional path prefix for the EKM proxy (if any).
    :keyword str server_subject_common_name: The subject common name of the server certificate of
        the EKM proxy.

    :ivar str host: EKM proxy FQDN.
    :ivar list[bytes] server_ca_certificates: The root CA certificate chain that issued the proxy
        server's certificate.
    :ivar path_prefix: Optional path prefix for the EKM proxy (if any).
    :vartype path_prefix: str or None
    :ivar server_subject_common_name: The subject common name of the server certificate of the EKM
        proxy.
    :vartype server_subject_common_name: str or None
    """

    def __init__(
        self,
        host: str,
        server_ca_certificates: List[bytes],
        *,
        path_prefix: Optional[str] = None,
        server_subject_common_name: Optional[str] = None,
    ) -> None:
        self.host = host
        self.server_ca_certificates = server_ca_certificates
        self.path_prefix = path_prefix
        self.server_subject_common_name = server_subject_common_name

    def __repr__(self) -> str:
        return f"KeyVaultEkmConnection<{self.host}>"

    @classmethod
    def _from_generated(cls, connection: EkmConnection) -> "KeyVaultEkmConnection":
        return cls(
            host=connection.host,
            server_ca_certificates=connection.server_ca_certificates,
            path_prefix=connection.path_prefix,
            server_subject_common_name=connection.server_subject_common_name,
        )

    def _to_generated(self) -> EkmConnection:
        return EkmConnection(
            host=self.host,
            server_ca_certificates=self.server_ca_certificates,
            path_prefix=self.path_prefix,
            server_subject_common_name=self.server_subject_common_name,
        )


class KeyVaultEkmProxyClientCertificateInfo(object):
    """EKM proxy client certificate information.

    :ivar ca_certificates: The client root CA certificate chain to authenticate to the EKM proxy.
        An array of certificates in the certificate chain, each in DER format and base64 encoded.
    :vartype ca_certificates: list[bytes] or None
    :ivar subject_common_name: The subject common name of the client certificate used to
        authenticate to the EKM proxy.
    :vartype subject_common_name: str or None
    """

    def __init__(self, **kwargs: Any) -> None:
        self.ca_certificates: Optional[List[bytes]] = kwargs.get("ca_certificates")
        self.subject_common_name: Optional[str] = kwargs.get("subject_common_name")

    def __repr__(self) -> str:
        return f"KeyVaultEkmProxyClientCertificateInfo<{self.subject_common_name}>"

    @classmethod
    def _from_generated(
        cls, certificate_info: EkmProxyClientCertificateInfo
    ) -> "KeyVaultEkmProxyClientCertificateInfo":
        return cls(
            ca_certificates=certificate_info.ca_certificates,
            subject_common_name=certificate_info.subject_common_name,
        )


class KeyVaultEkmProxyInfo(object):
    """EKM proxy information returned when checking an EKM connection.

    :ivar api_version: The highest version of proxy interface API supported by the EKM proxy.
    :vartype api_version: str or None
    :ivar proxy_vendor: The name of the proxy vendor.
    :vartype proxy_vendor: str or None
    :ivar proxy_name: The name of the proxy product and its version.
    :vartype proxy_name: str or None
    :ivar ekm_vendor: The name of the EKM vendor.
    :vartype ekm_vendor: str or None
    :ivar ekm_product: The name of the EKM product and its version.
    :vartype ekm_product: str or None
    """

    def __init__(self, **kwargs: Any) -> None:
        self.api_version: Optional[str] = kwargs.get("api_version")
        self.proxy_vendor: Optional[str] = kwargs.get("proxy_vendor")
        self.proxy_name: Optional[str] = kwargs.get("proxy_name")
        self.ekm_vendor: Optional[str] = kwargs.get("ekm_vendor")
        self.ekm_product: Optional[str] = kwargs.get("ekm_product")

    def __repr__(self) -> str:
        return (
            f"KeyVaultEkmProxyInfo(api_version={self.api_version}, "
            f"proxy_vendor={self.proxy_vendor}, proxy_name={self.proxy_name}, "
            f"ekm_vendor={self.ekm_vendor}, ekm_product={self.ekm_product})"
        )

    @classmethod
    def _from_generated(cls, proxy_info: EkmProxyInfo) -> "KeyVaultEkmProxyInfo":
        return cls(
            api_version=proxy_info.api_version,
            proxy_vendor=proxy_info.proxy_vendor,
            proxy_name=proxy_info.proxy_name,
            ekm_vendor=proxy_info.ekm_vendor,
            ekm_product=proxy_info.ekm_product,
        )



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import os
import time
import uuid

import pytest
from azure.keyvault.administration import KeyVaultDataAction, KeyVaultPermission, KeyVaultRoleScope
from devtools_testutils import add_general_regex_sanitizer, recorded_by_proxy, set_bodiless_matcher

from _shared.test_case import KeyVaultTestCase
from _test_case import KeyVaultAccessControlClientPreparer, get_decorator

all_api_versions = get_decorator()


class TestAccessControl(KeyVaultTestCase):
    def get_replayable_uuid(self, replay_value):
        if self.is_live:
            value = str(uuid.uuid4())
            return value
        return replay_value

    def get_service_principal_id(self):
        replay_value = "service-principal-id"
        if self.is_live:
            value = os.environ.get("MANAGED_IDENTITY_CLIENT_ID")
            return value
        return replay_value

    @pytest.mark.parametrize("api_version", all_api_versions)
    @KeyVaultAccessControlClientPreparer()
    @recorded_by_proxy
    def test_role_definitions(self, client, **kwargs):
        set_bodiless_matcher()
        # list initial role definitions
        scope = KeyVaultRoleScope.GLOBAL
        original_definitions = [d for d in client.list_role_definitions(scope)]
        assert len(original_definitions)

        # create custom role definition
        role_name = self.get_resource_name("role-name")
        definition_name = self.get_replayable_uuid("definition-name")
        add_general_regex_sanitizer(function_scoped=True, regex=definition_name, value="definition-name")

        permissions = [KeyVaultPermission(data_actions=[KeyVaultDataAction.READ_HSM_KEY])]
        created_definition = client.set_role_definition(
            scope=scope, name=definition_name, role_name=role_name, description="test", permissions=permissions
        )
        assert "/" in created_definition.assignable_scopes
        assert created_definition.role_name == role_name
        assert created_definition.name == definition_name
        assert created_definition.description == "test"
        assert len(created_definition.permissions) == 1
        assert created_definition.permissions[0].data_actions == [KeyVaultDataAction.READ_HSM_KEY]
        assert created_definition.assignable_scopes == [KeyVaultRoleScope.GLOBAL]
        # update custom role definition
        permissions = [KeyVaultPermission(data_actions=[], not_data_actions=[KeyVaultDataAction.READ_HSM_KEY])]
        role_name2 = self.get_resource_name("role-name2")
        updated_definition = client.set_role_definition(
            scope=scope, name=definition_name, role_name=role_name2, permissions=permissions
        )
        assert updated_definition.role_name == role_name2
        assert updated_definition.description == ""
        assert len(updated_definition.permissions) == 1
        assert len(updated_definition.permissions[0].data_actions) == 0
        assert updated_definition.permissions[0].not_data_actions == [KeyVaultDataAction.READ_HSM_KEY]
        assert updated_definition.assignable_scopes == [KeyVaultRoleScope.GLOBAL]

        # assert that the created role definition isn't duplicated
        matching_definitions = [d for d in client.list_role_definitions(scope) if d.id == updated_definition.id]
        assert len(matching_definitions) == 1

        # get custom role definition
        definition = client.get_role_definition(scope=scope, name=definition_name)
        assert_role_definitions_equal(definition, updated_definition)

        # delete custom role definition
        client.delete_role_definition(scope, definition_name)

        assert not any(d.id == definition.id for d in client.list_role_definitions(scope))
        if self.is_live:
            time.sleep(60)  # additional waiting to avoid conflicts with resources in other tests

    @pytest.mark.parametrize("api_version", all_api_versions)
    @KeyVaultAccessControlClientPreparer()
    @recorded_by_proxy
    def test_role_assignment(self, client, **kwargs):
        set_bodiless_matcher()
        scope = KeyVaultRoleScope.GLOBAL
        definitions = [d for d in client.list_role_definitions(scope)]

        # assign an arbitrary role to the service principal authenticating these requests
        definition = definitions[0]
        principal_id = self.get_service_principal_id()
        name = self.get_replayable_uuid("some-uuid")
        add_general_regex_sanitizer(function_scoped=True, regex=name, value="some-uuid")

        created = client.create_role_assignment(scope, definition.id, principal_id, name=name)
        assert created.name == name
        # assert created.properties.principal_id == principal_id
        assert created.properties.role_definition_id == definition.id
        assert created.properties.scope == scope

        # should be able to get the new assignment
        got = client.get_role_assignment(scope, name)
        assert got.name == name
        # assert got.properties.principal_id == principal_id
        assert got.properties.role_definition_id == definition.id
        assert got.properties.scope == scope

        # new assignment should be in the list of all assignments
        matching_assignments = [
            a for a in client.list_role_assignments(scope) if a.role_assignment_id == created.role_assignment_id
        ]
        assert len(matching_assignments) == 1

        # delete the assignment
        client.delete_role_assignment(scope, created.name)

        assert not any(a.role_assignment_id == created.role_assignment_id for a in client.list_role_assignments(scope))
        if self.is_live:
            time.sleep(60)  # additional waiting to avoid conflicts with resources in other tests


def assert_role_definitions_equal(d1, d2):
    assert d1.id == d2.id
    assert d1.name == d2.name
    assert d1.role_name == d2.role_name
    assert d1.description == d2.description
    assert d1.role_type == d2.role_type
    assert d1.type == d2.type
    assert len(d1.permissions) == len(d2.permissions)
    for i in range(len(d1.permissions)):
        assert d1.permissions[i].actions == d2.permissions[i].actions
        assert d1.permissions[i].not_actions == d2.permissions[i].not_actions
        assert d1.permissions[i].data_actions == d2.permissions[i].data_actions
        assert d1.permissions[i].not_data_actions == d2.permissions[i].not_data_actions
    assert d1.assignable_scopes == d2.assignable_scopes



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import base64
import pytest

from azure.core.exceptions import HttpResponseError
from azure.keyvault.administration import KeyVaultEkmClient, KeyVaultEkmConnection
from azure.keyvault.administration._internal.client_base import DEFAULT_VERSION

from devtools_testutils import recorded_by_proxy

from _shared.test_case import KeyVaultTestCase
from _test_case import KeyVaultEkmClientPreparer, get_decorator

only_latest = get_decorator(api_versions=[DEFAULT_VERSION])

# Note: These tests require an EKM connection to be established with an EKM Sample Proxy.


class TestEkm(KeyVaultTestCase):
    @pytest.mark.parametrize("api_version", only_latest)
    @KeyVaultEkmClientPreparer()
    @recorded_by_proxy
    def test_ekm_connection(self, client: KeyVaultEkmClient, **kwargs):
        ekm_host = kwargs.pop("ekm_host")
        server_ca_certificate = kwargs.pop("ekm_certificate")
        if not server_ca_certificate or not ekm_host:
            pytest.skip(
                "EKM CA certificate is required for live tests. Please set the EKM_PROXY_HOST and EKM_SERVER_CA_CERTIFICATE environment variables."
            )

        # Cleanup
        try:
            client.delete_ekm_connection()
        except HttpResponseError:
            pass

        # Create an EKM connection
        ekm_connection = KeyVaultEkmConnection(
            host=ekm_host,
            server_ca_certificates=[base64.b64decode(server_ca_certificate)],
            path_prefix="/api/v1",
        )
        created_ekm_connection = client.create_ekm_connection(connection=ekm_connection)
        assert created_ekm_connection is not None
        assert created_ekm_connection.host == ekm_host
        assert created_ekm_connection.server_ca_certificates is not None
        assert len(created_ekm_connection.server_ca_certificates) == 1
        assert created_ekm_connection.path_prefix == ekm_connection.path_prefix
        assert created_ekm_connection.server_subject_common_name == ekm_connection.server_subject_common_name

        # Get the EKM connection
        retrieved_ekm_connection = client.get_ekm_connection()
        assert retrieved_ekm_connection is not None
        assert retrieved_ekm_connection.host == ekm_host
        assert retrieved_ekm_connection.server_ca_certificates is not None
        assert len(retrieved_ekm_connection.server_ca_certificates) == 1
        assert retrieved_ekm_connection.path_prefix == ekm_connection.path_prefix
        assert retrieved_ekm_connection.server_subject_common_name == created_ekm_connection.server_subject_common_name

        # Get the EKM certificate
        ekm_certificate = client.get_ekm_certificate()
        assert ekm_certificate is not None
        assert ekm_certificate.ca_certificates is not None
        assert len(ekm_certificate.ca_certificates) == 1

        # Check the EKM connection status
        connection_status = client.check_ekm_connection()
        assert connection_status is not None
        assert connection_status.api_version is not None
        assert connection_status.proxy_vendor is not None
        assert connection_status.proxy_name is not None
        assert connection_status.ekm_vendor is not None
        assert connection_status.ekm_product is not None

        # Update the EKM connection
        updated_ekm_connection = KeyVaultEkmConnection(
            host=ekm_host,
            server_ca_certificates=[base64.b64decode(server_ca_certificate)],
            path_prefix="/api/v1",
        )
        result = client.update_ekm_connection(connection=updated_ekm_connection)
        assert result is not None
        assert result.host == updated_ekm_connection.host
        assert result.server_ca_certificates is not None
        assert len(result.server_ca_certificates) == 1
        assert result.path_prefix == updated_ekm_connection.path_prefix
        assert result.server_subject_common_name == updated_ekm_connection.server_subject_common_name

        # Delete the EKM connection
        result = client.delete_ekm_connection()
        assert result is not None
        assert result.host == updated_ekm_connection.host
        assert result.server_ca_certificates is not None
        assert len(result.server_ca_certificates) == 1
        assert result.path_prefix == updated_ekm_connection.path_prefix
        assert result.server_subject_common_name == updated_ekm_connection.server_subject_common_name
